"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const uuid_1 = require("uuid");
const socket_1 = require("./socket");
const logger_1 = require("../logs/logger");
const forwardHttpRequest_1 = require("../common/relay/forwardHttpRequest");
const webserver_1 = require("../common/http/webserver");
const version_1 = __importDefault(require("../common/utils/version"));
const checks_handler_1 = require("./checks/api/checks-handler");
const healthcheckHandler_1 = require("./routesHandler/healthcheckHandler");
const systemCheckHandler_1 = require("./routesHandler/systemCheckHandler");
const processHooks_1 = require("./hooks/startup/processHooks");
const forwardHttpRequestOverHttp_1 = require("../common/relay/forwardHttpRequestOverHttp");
const socketHelpers_1 = require("./utils/socketHelpers");
const filtersAsync_1 = require("../common/filter/filtersAsync");
const websocketConnectionMiddlewares_1 = require("./routesHandler/websocketConnectionMiddlewares");
process.on('uncaughtException', (error) => {
    if (error.message == 'read ECONNRESET') {
        logger_1.log.error({ msg: error.message, stackTrace: error.stack }, 'ECONNRESETs Catch all:', error.message);
    }
    else {
        logger_1.log.error({ msg: error.message, stackTrace: error.stack }, 'Uncaught exception:', error.message);
        process.exit(1);
    }
});
const main = async (clientOpts) => {
    try {
        logger_1.log.info({ version: version_1.default }, 'running in client mode');
        const brokerClientId = (0, uuid_1.v4)();
        logger_1.log.info({ brokerClientId }, 'generated broker client id');
        const hookResults = await (0, processHooks_1.processStartUpHooks)(clientOpts, brokerClientId);
        const loadedClientOpts = {
            loadedFilters: (0, filtersAsync_1.loadAllFilters)(clientOpts.filters, clientOpts.config),
            ...clientOpts,
        };
        if (!loadedClientOpts.loadedFilters) {
            logger_1.log.error({ clientOpts }, 'Unable to load filters');
            throw new Error('Unable to load filters');
        }
        const globalIdentifyingMetadata = {
            capabilities: ['post-streams'],
            clientId: brokerClientId,
            filters: clientOpts.filters,
            preflightChecks: hookResults.preflightCheckResults,
            version: version_1.default,
        };
        let websocketConnections = [];
        if (clientOpts.config.universalBrokerEnabled) {
            websocketConnections = (0, socket_1.createWebSockets)(loadedClientOpts, globalIdentifyingMetadata);
        }
        else {
            websocketConnections.push((0, socket_1.createWebSocket)(loadedClientOpts, globalIdentifyingMetadata));
        }
        // start the local webserver to listen for relay requests
        const { app, server } = (0, webserver_1.webserver)(clientOpts.config, clientOpts.port);
        const httpToWsForwarder = (0, forwardHttpRequest_1.forwardHttpRequest)(loadedClientOpts);
        const httpToAPIForwarder = (0, forwardHttpRequestOverHttp_1.forwardHttpRequestOverHttp)(loadedClientOpts, clientOpts.config);
        // IMPORTANT: defined before relay (`app.all('/*', ...`)
        app.get('/health/checks', (0, checks_handler_1.handleChecksRoute)(clientOpts.config));
        app.get('/health/checks/:checkId', (0, checks_handler_1.handleCheckIdsRoutes)(clientOpts.config));
        app.get(clientOpts.config.brokerHealthcheckPath || '/healthcheck', (req, res, next) => {
            res.locals.websocketConnections = websocketConnections;
            next();
        }, healthcheckHandler_1.healthCheckHandler);
        app.get('/filters', (req, res) => {
            res.send(loadedClientOpts.filters);
        });
        app.get(clientOpts.config.brokerSystemcheckPath || '/systemcheck', (req, res, next) => {
            res.locals.clientOpts = loadedClientOpts;
            next();
        }, systemCheckHandler_1.systemCheckHandler);
        app.post('/webhook/*', (req, res, next) => {
            res.locals.websocketConnections = websocketConnections;
            next();
        }, websocketConnectionMiddlewares_1.websocketConnectionSelectorMiddleware, (req, res) => {
            if ((0, socketHelpers_1.isWebsocketConnOpen)(res.locals.websocket)) {
                httpToWsForwarder(req, res);
            }
            else {
                logger_1.log.warn('Websocket connection closed, forwarding via API');
                httpToAPIForwarder(req, res);
            }
        });
        // relay all other URL paths
        app.all('/*', (req, res, next) => {
            // Middleware checks the request url for /webhook/scmType/ID, and from the scmType pick the relevant filter.
            // Need to check what CRA agent requests look like so we don't mess that up.
            // New hooks we want to inject the BROKER_CLIENT_URL with the integration ID so we can select based on the integration ID
            // but doing so can't break the existing webhooks
            res.locals.websocketConnections = websocketConnections;
            next();
        }, websocketConnectionMiddlewares_1.websocketConnectionSelectorMiddleware, httpToWsForwarder);
        return {
            websocketConnections,
            close: (done) => {
                logger_1.log.info('client websocket is closing');
                server.close();
                for (let i = 0; i < websocketConnections.length; i++) {
                    websocketConnections[i].destroy(function () {
                        logger_1.log.info(`client websocket ${websocketConnections[i].identifier || ''} is closed`);
                        if (done) {
                            return done();
                        }
                    });
                }
            },
        };
    }
    catch (err) {
        logger_1.log.warn({ err }, `Shutting down client`);
        throw err;
    }
};
exports.main = main;
//# sourceMappingURL=index.js.map