"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.systemCheckHandler = void 0;
const credentials_1 = require("../utils/credentials");
const logger_1 = require("../../logs/logger");
const config_1 = require("../../common/config/config");
const connectionValidation_1 = require("../utils/connectionValidation");
const universal_1 = require("../../common/config/universal");
const systemCheckHandler = async (req, res) => {
    // Systemcheck is the broker client's ability to assert the network
    // reachability and some correctness of credentials for the service
    // being proxied by the broker client.
    const clientOpts = res.locals.clientOpts;
    if (clientOpts.config.universalBrokerEnabled) {
        const configForAllConnections = (0, universal_1.getConfigForConnections)();
        const validationResults = [];
        let isValidationPassing = true;
        for (const [connectionName, config] of configForAllConnections.entries()) {
            config.validations = (0, config_1.expandConfigObjectRecursively)(config.validations, config);
            logger_1.log.info({ connectionName }, '[System Check] Validating connection');
            const { passing, data } = await (0, connectionValidation_1.validateConnection)(config);
            const result = {
                connectionName: connectionName,
                validated: passing,
                results: data,
            };
            if (!passing) {
                result['message'] = `Validation failed, please review connection details for ${connectionName}`;
                isValidationPassing = false;
            }
            validationResults.push(result);
        }
        res.status(isValidationPassing ? 200 : 500).send(validationResults);
    }
    else {
        const brokerClientValidationMethod = clientOpts.config.brokerClientValidationMethod || 'GET';
        const brokerClientValidationTimeoutMs = clientOpts.config.brokerClientValidationTimeoutMs || 5000;
        // set auth header according to config
        const { auths, rawCreds } = (0, credentials_1.loadCredentialsFromConfig)(clientOpts.config);
        // make the internal validation request
        const validationResults = [];
        let errorOccurred = true;
        if (auths.length > 0) {
            for (let i = 0; i < auths.length; i++) {
                logger_1.log.info(`Checking if credentials at index ${i} are valid`);
                const auth = auths[i];
                const rawCred = rawCreds[i];
                const { data, errorOccurred: err } = await (0, credentials_1.checkCredentials)(auth, clientOpts.config, brokerClientValidationMethod, brokerClientValidationTimeoutMs);
                data['maskedCredentials'] =
                    rawCred.length <= 6
                        ? '***'
                        : `${rawCred.substring(0, 3)}***${rawCred.substring(rawCred.length - 3)}`;
                validationResults.push(data);
                errorOccurred = err;
                logger_1.log.info('Credentials checked');
            }
        }
        else {
            logger_1.log.info('No credentials specified - checking if target can be accessed without credentials');
            const { data, errorOccurred: err } = await (0, credentials_1.checkCredentials)(null, clientOpts.config, brokerClientValidationMethod, brokerClientValidationTimeoutMs);
            data['maskedCredentials'] = null;
            validationResults.push(data);
            errorOccurred = err;
        }
        if (errorOccurred) {
            return res.status(500).json(validationResults);
        }
        else {
            return res.status(200).json(validationResults);
        }
    }
};
exports.systemCheckHandler = systemCheckHandler;
//# sourceMappingURL=systemCheckHandler.js.map