"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.expandConfigObjectRecursively = exports.expandPlaceholderValuesInFlatList = exports.loadBrokerConfig = exports.findProjectRoot = exports.getConfig = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const camelcase_1 = __importDefault(require("camelcase"));
const snyk_config_1 = require("snyk-config");
const dotenv_1 = __importDefault(require("dotenv"));
const logger_1 = require("../../logs/logger");
let config = {};
const getConfig = () => {
    return config;
};
exports.getConfig = getConfig;
const findProjectRoot = (startDir) => {
    let currentDir = startDir;
    while (currentDir !== '/') {
        const snykConfigPath = path_1.default.join(currentDir, 'config.default.json');
        if (fs_1.default.existsSync(snykConfigPath)) {
            return currentDir;
        }
        currentDir = path_1.default.dirname(currentDir);
    }
    return null;
};
exports.findProjectRoot = findProjectRoot;
const loadBrokerConfig = (localConfigForTest) => {
    var _a;
    dotenv_1.default.config({
        path: path_1.default.join(process.cwd(), '.env'),
    });
    try {
        const localConfig = localConfigForTest
            ? localConfigForTest
            : (0, snyk_config_1.loadConfig)((_a = (0, exports.findProjectRoot)(__dirname)) !== null && _a !== void 0 ? _a : process.cwd());
        expand(process.env);
        config = Object.assign({}, camelify(localConfig), camelify(process.env));
        // for each in config.brokerClientConfiguration.common.default check if process env exist and if it does,
        // override config.brokerClientConfiguration.common.default
        // clientOpts.config.brokerClientConfiguration.common.default.BROKER_SERVER_URL = process.env.BROKER_SERVER_URL
        if (config.universalBrokerEnabled) {
            config = getConsolidatedConfigForUniversalBroker(config);
        }
    }
    catch (error) {
        logger_1.log.error({ error }, 'Error loading configuration');
        const refError = new ReferenceError('Error loading configuration');
        refError['code'] = 'UNABLE_TO_LOAD_CONFIGURATION';
        throw error;
    }
    if (config.caCert) {
        config.caCert = fs_1.default.readFileSync(path_1.default.resolve(process.cwd(), config.caCert));
    }
    for (const [key, value] of Object.entries(config)) {
        if ((key.endsWith('Pool') || key.endsWith('_POOL')) &&
            !Array.isArray(value)) {
            config[key] = value.split(',').map((s) => s.trim());
        }
    }
};
exports.loadBrokerConfig = loadBrokerConfig;
const getConsolidatedConfigForUniversalBroker = (configToConsolidate) => {
    const commonDefaultConfigKeys = Object.keys(configToConsolidate.brokerClientConfiguration.common.default);
    for (let i = 0; i < commonDefaultConfigKeys.length; i++) {
        // override default values with any env var override for the common settings
        configToConsolidate.brokerClientConfiguration.common.default[commonDefaultConfigKeys[i]] =
            process.env[commonDefaultConfigKeys[i]] ||
                configToConsolidate.brokerClientConfiguration.common.default[commonDefaultConfigKeys[i]];
        // Moving common settings only to config level like classic broker
        configToConsolidate[commonDefaultConfigKeys[i]] =
            configToConsolidate.brokerClientConfiguration.common.default[commonDefaultConfigKeys[i]];
    }
    // share the broker client url across the connections requiring it
    if (configToConsolidate.brokerClientUrl) {
        const brokerClientConfigurationKeys = Object.keys(configToConsolidate.brokerClientConfiguration);
        for (const key of brokerClientConfigurationKeys) {
            if (configToConsolidate.brokerClientConfiguration[key].required
                .BROKER_CLIENT_URL) {
                configToConsolidate.brokerClientConfiguration[key].default.BROKER_CLIENT_URL = configToConsolidate.brokerClientUrl;
            }
        }
    }
    return camelify(configToConsolidate);
};
function camelify(res) {
    return Object.keys(res).reduce((acc, key) => {
        const camelKey = (0, camelcase_1.default)(key);
        acc[camelKey] = res[key];
        acc[key] = res[key];
        return acc;
    }, {});
}
function expandValue(obj, value) {
    let poolFound = undefined;
    let keyWithPool = undefined;
    const variableRegex = /(\\?\$.+?\b)/g;
    const variableMatcher = value.match(variableRegex);
    if (variableMatcher) {
        for (const key of variableMatcher) {
            if (key[0] === '$' && obj[key.slice(1) + '_POOL']) {
                keyWithPool = key.slice(1);
                poolFound = key.slice(1) + '_POOL';
                break;
            }
        }
    }
    if (poolFound) {
        const values = [];
        let pool;
        if (Array.isArray(obj[poolFound])) {
            pool = obj[poolFound];
        }
        else {
            pool = obj[poolFound].split(',').map((s) => s.trim());
            obj[poolFound] = pool;
        }
        for (const o of pool) {
            values.push(value.replace(variableRegex, (all, key) => {
                if (key[0] === '$') {
                    const keyToReplace = key.slice(1);
                    return keyToReplace === keyWithPool ? o : obj[keyToReplace] || key;
                }
                return key;
            }));
        }
        return values;
    }
    else {
        return value.replace(variableRegex, (all, key) => {
            if (key[0] === '$') {
                const keyToReplace = key.slice(1);
                return obj[keyToReplace] || key;
            }
            return key;
        });
    }
}
function expand(obj) {
    const keys = Object.keys(obj);
    for (const key of keys) {
        const value = expandValue(obj, obj[key]);
        if (value && Array.isArray(value)) {
            obj[key + '_POOL'] = value;
        }
        else if (value !== obj[key]) {
            obj[key] = value;
        }
    }
    return obj;
}
const expandPlaceholderValuesInFlatList = (objectToExpand, referenceConfig) => {
    for (const key of Object.keys(objectToExpand)) {
        if (typeof objectToExpand[key] == 'string' &&
            objectToExpand[key].indexOf('$') > -1) {
            const regex = /\$([a-zA-Z0-9_]+)/;
            const match = regex.exec(objectToExpand[key]);
            const extractedEnvVarName = match ? match[1] : '';
            if (extractedEnvVarName) {
                objectToExpand[key] = objectToExpand[key].replace(`$${extractedEnvVarName}`, referenceConfig[extractedEnvVarName]);
            }
        }
    }
    return objectToExpand;
};
exports.expandPlaceholderValuesInFlatList = expandPlaceholderValuesInFlatList;
const expandConfigObjectRecursively = (objToExpand, referenceConfig) => {
    for (const key of Object.keys(objToExpand)) {
        if (typeof objToExpand[key] == 'string') {
            objToExpand[key] = expandValue(referenceConfig, objToExpand[key]);
        }
        else {
            objToExpand[key] = (0, exports.expandConfigObjectRecursively)(objToExpand[key], referenceConfig);
        }
    }
    return objToExpand;
};
exports.expandConfigObjectRecursively = expandConfigObjectRecursively;
//# sourceMappingURL=config.js.map