"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadFilters = exports.loadAllFilters = void 0;
const minimatch_1 = __importDefault(require("minimatch"));
const path_to_regexp_1 = __importDefault(require("path-to-regexp"));
const qs_1 = __importDefault(require("qs"));
const path_1 = __importDefault(require("path"));
const undefsafe_1 = __importDefault(require("undefsafe"));
const replace_vars_1 = require("../utils/replace-vars");
const auth_header_1 = __importDefault(require("../utils/auth-header"));
const try_json_parse_1 = __importDefault(require("../utils/try-json-parse"));
const logger_1 = require("../../logs/logger");
const utils_1 = require("./utils");
const universal_1 = require("../config/universal");
const loadAllFilters = (filters, configFromApp) => {
    const filtersKeys = Object.keys(filters);
    if (filtersKeys.includes('public')) {
        const classicFilters = filters;
        return {
            public: (0, exports.loadFilters)(classicFilters.public, 'default', configFromApp),
            private: (0, exports.loadFilters)(classicFilters.private, 'default', configFromApp),
        };
    }
    else {
        const filtersMap = new Map();
        for (let i = 0; i < filtersKeys.length; i++) {
            logger_1.log.info({ type: filtersKeys[i] }, `Loading ruleset (private + public rules) for ${filtersKeys[i]}`);
            const loadedFilterSet = {
                public: (0, exports.loadFilters)(filters[filtersKeys[i]].public, filtersKeys[i], configFromApp),
                private: (0, exports.loadFilters)(filters[filtersKeys[i]].private, filtersKeys[i], configFromApp),
            };
            filtersMap.set(filtersKeys[i], loadedFilterSet);
            logger_1.log.info({ type: filtersKeys[i] }, `Loaded ${filters[filtersKeys[i]].private.length} private & ${filters[filtersKeys[i]].public.length} public rules`);
        }
        return filtersMap;
    }
};
exports.loadAllFilters = loadAllFilters;
// reads config that defines
const loadFilters = (ruleSource, type, configFromApp) => {
    let rules = [];
    // polymorphic support
    if (Array.isArray(ruleSource)) {
        rules = ruleSource;
    }
    else if (ruleSource) {
        logger_1.log.error({ ruleSource }, 'Unable to parse rule source, ignoring');
        throw new Error(`Expected array of filter rules, got '${typeof rules}' instead.`);
    }
    logger_1.log.debug({ rulesCount: rules.length }, `loading new rules ${type ? 'for ' + type + '.' : '.'}`);
    // array of entries with
    const tests = rules.map((entry) => {
        const keys = [];
        let { method, origin, // eslint-disable-line prefer-const
        path: entryPath, valid, // eslint-disable-line prefer-const
        requiredCapabilities, // eslint-disable-line prefer-const
         } = entry;
        const baseOrigin = origin;
        const { stream } = entry;
        method = (method || 'get').toLowerCase();
        const bodyFilters = valid ? valid.filter((v) => !!v.path && !v.regex) : [];
        const bodyRegexFilters = valid
            ? valid.filter((v) => !!v.path && !!v.regex)
            : [];
        const queryFilters = valid ? valid.filter((v) => !!v.queryParam) : [];
        const headerFilters = valid ? valid.filter((v) => !!v.header) : [];
        // now track if there's any values that we need to interpolate later
        const fromConfig = {};
        // load config from config.default.json based on type and config.universal.json based on token
        let localConfig = type && (configFromApp === null || configFromApp === void 0 ? void 0 : configFromApp.universalBrokerEnabled)
            ? Object.assign({}, (0, universal_1.getConfigForType)(type), configFromApp)
            : configFromApp;
        // slightly bespoke version of replace-vars.js
        entryPath = (entryPath || '').replace(/(\${.*?})/g, (_, match) => {
            const key = match.slice(2, -1); // ditch the wrappers
            fromConfig[key] = localConfig[key] || '';
            return ':' + key;
        });
        if (entryPath[0] !== '/') {
            entryPath = '/' + entryPath;
        }
        logger_1.log.debug({ method, path: entryPath }, 'adding new filter rule');
        const regexp = (0, path_to_regexp_1.default)(entryPath, keys);
        return (req) => {
            var _a, _b;
            if ((configFromApp === null || configFromApp === void 0 ? void 0 : configFromApp.brokerType) === 'client' &&
                (configFromApp === null || configFromApp === void 0 ? void 0 : configFromApp.universalBrokerEnabled) &&
                req.connectionIdentifier) {
                localConfig = (0, universal_1.overloadConfigWithConnectionSpecificConfig)(req.connectionIdentifier, localConfig);
            }
            // check the request method
            if (req.method.toLowerCase() !== method && method !== 'any') {
                return false;
            }
            // Do not allow directory traversal
            if (path_1.default.normalize(req.url) !== req.url) {
                return false;
            }
            // Discard any fragments before further processing
            const mainURI = req.url.split('#')[0];
            // query params might contain additional "?"s, only split on the 1st one
            const parts = mainURI.split('?');
            let [url, querystring] = [parts[0], parts.slice(1).join('?')];
            const res = regexp.exec(url);
            if (!res) {
                // no url match
                return false;
            }
            // reconstruct the url from the user config
            for (let i = 1; i < res.length; i++) {
                const val = fromConfig[keys[i - 1].name];
                if (val) {
                    url = url.replace(res[i], val);
                }
            }
            // if validity filters are present, at least one must be satisfied
            if (bodyFilters.length ||
                bodyRegexFilters.length ||
                queryFilters.length) {
                let isValid;
                let parsedBody;
                if (bodyFilters.length) {
                    parsedBody = (0, try_json_parse_1.default)(req.body);
                    // validate against the body
                    isValid = bodyFilters.some(({ path: filterPath, value }) => {
                        return (0, undefsafe_1.default)(parsedBody, filterPath, value);
                    });
                }
                if (!isValid && bodyRegexFilters.length) {
                    parsedBody = parsedBody || (0, try_json_parse_1.default)(req.body);
                    // validate against the body by regex
                    isValid = bodyRegexFilters.some(({ path: filterPath, regex }) => {
                        try {
                            const re = new RegExp(regex); //paths without regexes got filtered out earlier, hence the !
                            return re.test((0, undefsafe_1.default)(parsedBody, filterPath));
                        }
                        catch (error) {
                            logger_1.log.error({ error, path: filterPath, regex }, 'failed to test regex rule');
                            return false;
                        }
                    });
                }
                // no need to check query filters if the request is already valid
                if (!isValid && queryFilters.length) {
                    const parsedQuerystring = qs_1.default.parse(querystring);
                    // validate against the querystring
                    isValid = queryFilters.every(({ queryParam, values }) => {
                        // ! because the value is not undefined, queryFilters length to be non zero and array filtered earlier
                        return values.some((value) => 
                        // ! because the queryParam is not undefined, queryFilters length to be non zero and array filtered earlier
                        (0, minimatch_1.default)(parsedQuerystring[queryParam] || '', value, {
                            dot: true,
                        }));
                    });
                }
                if (!isValid) {
                    return false;
                }
            }
            if (headerFilters.length) {
                if (!(0, utils_1.validateHeaders)(headerFilters, req.headers)) {
                    return false;
                }
            }
            if (requiredCapabilities) {
                let matchedAll = true;
                for (const c of requiredCapabilities) {
                    if (!((_a = req === null || req === void 0 ? void 0 : req.locals) === null || _a === void 0 ? void 0 : _a.capabilities.includes(c))) {
                        matchedAll = false;
                        logger_1.log.warn({
                            path: entryPath,
                            capability: c,
                            clientCapabilities: (_b = req === null || req === void 0 ? void 0 : req.locals) === null || _b === void 0 ? void 0 : _b.capabilities,
                        }, 'client does not report support for capability');
                    }
                }
                if (!matchedAll) {
                    // We have to throw to avoid it getting approved by a generic matcher later on
                    throw new Error('client does not support all required capabilities for endpoint');
                }
            }
            const origin = (0, replace_vars_1.replace)(baseOrigin, localConfig);
            logger_1.log.debug({ path: entryPath, origin, url, querystring }, 'rule matched');
            querystring = querystring ? `?${querystring}` : '';
            return {
                url: origin + url + querystring,
                auth: entry.auth && (0, auth_header_1.default)(entry.auth),
                stream,
            };
        };
    });
    return (payload) => {
        let res = false;
        logger_1.log.debug({ rulesCount: tests.length }, 'looking for a rule match');
        try {
            for (const test of tests) {
                res = test(payload);
                if (res) {
                    break;
                }
            }
        }
        catch (e) {
            logger_1.log.warn({ error: e }, 'caught error checking request against rules');
            res = false;
        }
        return res;
    };
};
exports.loadFilters = loadFilters;
//# sourceMappingURL=filtersAsync.js.map