"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.forwardHttpRequestOverHttp = void 0;
const uuid_1 = require("uuid");
const logger_1 = require("../../logs/logger");
const metrics_1 = require("../utils/metrics");
const request_1 = require("../http/request");
const token_1 = require("../utils/token");
// 1. Request coming in over HTTP conn (logged)
// 2. Filter for rule match (log and block if no match)
// 3. Relay over websocket conn (logged)
// 4. Get response over websocket conn (logged)
// 5. Send response over HTTP conn
const forwardHttpRequestOverHttp = (options, config) => {
    // const filters = loadFilters(filterRules);
    return (req, res) => {
        var _a;
        var _b;
        // If this is the server, we should receive a Snyk-Request-Id header from upstream
        // If this is the client, we will have to generate one
        (_b = req.headers)['snyk-request-id'] || (_b['snyk-request-id'] = (0, uuid_1.v4)());
        const logContext = {
            url: req.url,
            requestMethod: req.method,
            requestHeaders: req.headers,
            requestId: req.headers['snyk-request-id'] &&
                Array.isArray(req.headers['snyk-request-id'])
                ? req.headers['snyk-request-id'].join(',')
                : req.headers['snyk-request-id'] || '',
            maskedToken: req['maskedToken'],
            hashedToken: req['hashedToken'],
        };
        const simplifiedContext = logContext;
        delete simplifiedContext.requestHeaders;
        logger_1.log.info(simplifiedContext, '[HTTP Flow] Received request');
        let filterResponse;
        if (options.config.brokerType == 'client' &&
            options.config.universalBrokerEnabled) {
            const clientOptions = options;
            const loadedFilters = clientOptions.loadedFilters;
            filterResponse =
                ((_a = loadedFilters
                    .get(res.locals.websocket.supportedIntegrationType) // The chosen type is determined by websocket connect middlwr
                ) === null || _a === void 0 ? void 0 : _a.public(req)) || false;
        }
        else {
            const loadedFilters = options.loadedFilters;
            filterResponse = loadedFilters.public(req);
        }
        if (!filterResponse) {
            (0, metrics_1.incrementHttpRequestsTotal)(true, 'inbound-request');
            const reason = 'Request does not match any accept rule, blocking HTTP request';
            logContext.error = 'blocked';
            logger_1.log.warn(logContext, reason);
            // TODO: respect request headers, block according to content-type
            return res.status(401).send({ message: 'blocked', reason, url: req.url });
        }
        else {
            (0, metrics_1.incrementHttpRequestsTotal)(false, 'inbound-request');
            const apiDomain = new URL(config.API_BASE_URL ||
                (config.BROKER_SERVER_URL
                    ? config.BROKER_SERVER_URL.replace('//broker.', '//api.')
                    : 'https://api.snyk.io'));
            const requestUri = new URL(req.url, apiDomain);
            req.headers['host'] = requestUri.host;
            req.headers['x-snyk-broker'] = `${(0, token_1.maskToken)(res.locals.websocket.identifier)}`;
            const filteredReq = {
                url: requestUri.toString(),
                method: req.method,
                body: req.body,
                headers: req.headers,
            };
            (0, request_1.makeRequestToDownstream)(filteredReq)
                .then((resp) => {
                if (resp.statusCode) {
                    res.status(resp.statusCode).set(resp.headers).send(resp.body);
                }
                else {
                    res.status(500).send(resp.statusText);
                }
            })
                .catch((err) => {
                logger_1.log.error(logContext, err, 'Failed to forward webhook event to Snyk Platform');
            });
        }
    };
};
exports.forwardHttpRequestOverHttp = forwardHttpRequestOverHttp;
//# sourceMappingURL=forwardHttpRequestOverHttp.js.map