"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.forwardWebSocketRequest = void 0;
const token_1 = require("../utils/token");
const logger_1 = require("../../logs/logger");
const downstream_post_stream_to_server_1 = require("../http/downstream-post-stream-to-server");
const metrics_1 = require("../utils/metrics");
const prepareRequest_1 = require("./prepareRequest");
const requestsHelper_1 = require("./requestsHelper");
const forwardWebSocketRequest = (options, websocketConnectionHandler) => {
    // 1. Request coming in over websocket conn (logged)
    // 2. Filter for rule match (log and block if no match)
    // 3. Relay over HTTP conn (logged)
    // 4. Get response over HTTP conn (logged)
    // 5. Send response over websocket conn
    return (connectionIdentifier) => async (payload, emit) => {
        var _a, _b, _c, _d, _e, _f;
        if (options.config.universalBrokerEnabled) {
            payload.connectionIdentifier = connectionIdentifier;
        }
        const requestId = payload.headers['snyk-request-id'];
        const logContext = {
            url: payload.url,
            connectionName: (_a = websocketConnectionHandler.friendlyName) !== null && _a !== void 0 ? _a : '',
            requestMethod: payload.method,
            requestHeaders: payload.headers,
            requestId,
            streamingID: payload.streamingID,
            maskedToken: (0, token_1.maskToken)(connectionIdentifier),
            hashedToken: (0, token_1.hashToken)(connectionIdentifier),
            transport: (_d = (_c = (_b = websocketConnectionHandler === null || websocketConnectionHandler === void 0 ? void 0 : websocketConnectionHandler.socket) === null || _b === void 0 ? void 0 : _b.transport) === null || _c === void 0 ? void 0 : _c.name) !== null && _d !== void 0 ? _d : 'unknown',
        };
        if (!requestId) {
            // This should be a warning but older clients won't send one
            // TODO make this a warning when significant majority of clients are on latest version
            logger_1.log.debug(logContext, 'Header Snyk-Request-Id not included in headers passed through');
        }
        // It clarifies which emit handler is for websockets vs POST response
        const websocketEmit = emit;
        const postOverrideEmit = (responseData, isResponseFromRequestModule = false) => {
            try {
                logContext.requestMethod = '';
                logContext.requestHeaders = {};
                const postHandler = new downstream_post_stream_to_server_1.BrokerServerPostResponseHandler(logContext, options.config, connectionIdentifier, options.config.universalBrokerEnabled
                    ? websocketConnectionHandler === null || websocketConnectionHandler === void 0 ? void 0 : websocketConnectionHandler.serverId
                    : options.config.serverId, requestId);
                if (isResponseFromRequestModule) {
                    logger_1.log.debug(logContext, '[Websocket Flow] Posting HTTP streaming response back to Broker Server');
                    postHandler.forwardRequest(responseData, payload.streamingID);
                }
                else {
                    logger_1.log.debug(logContext, '[Websocket Flow] Posting HTTP response back to Broker Server');
                    // Only for responses generated internally in the Broker Client/Server
                    postHandler.sendData(responseData, payload.streamingID);
                }
            }
            catch (err) {
                logger_1.log.error({ err }, `Error Posting via Emit callback.`);
            }
        };
        const legacyOverrideEmit = (responseData, isResponseFromRequestModule = false) => {
            // Traffic over websockets
            if (payload.streamingID) {
                if (isResponseFromRequestModule) {
                    (0, requestsHelper_1.legacyStreaming)(logContext, responseData, options.config, websocketConnectionHandler, payload.streamingID);
                }
                else {
                    websocketConnectionHandler === null || websocketConnectionHandler === void 0 ? void 0 : websocketConnectionHandler.send('chunk', payload.streamingID, '', false, {
                        status: responseData.status,
                        headers: responseData.headers,
                    });
                    websocketConnectionHandler === null || websocketConnectionHandler === void 0 ? void 0 : websocketConnectionHandler.send('chunk', payload.streamingID, JSON.stringify(responseData.body), true);
                }
            }
            else {
                logger_1.log.debug({ ...logContext, responseData }, '[Websocket Flow] (Legacy) Sending fixed response over WebSocket connection');
                websocketEmit(responseData);
            }
        };
        if ((_e = websocketConnectionHandler === null || websocketConnectionHandler === void 0 ? void 0 : websocketConnectionHandler.capabilities) === null || _e === void 0 ? void 0 : _e.includes('receive-post-streams')) {
            // Traffic over HTTP Post
            emit = postOverrideEmit;
        }
        else {
            emit = legacyOverrideEmit;
        }
        const simplifiedContext = logContext;
        delete simplifiedContext.requestHeaders;
        logger_1.log.info(simplifiedContext, `[Websocket Flow] Received request from ${process.env.BROKER_TYPE == 'client' ? 'server' : 'client'}`);
        let filterResponse;
        if (options.config.brokerType == 'client' &&
            options.config.universalBrokerEnabled) {
            const clientOptions = options;
            const loadedFilters = clientOptions.loadedFilters;
            filterResponse =
                ((_f = loadedFilters
                    .get(websocketConnectionHandler.supportedIntegrationType)) === null || _f === void 0 ? void 0 : _f.private(payload)) || false;
        }
        else {
            const loadedFilters = options.loadedFilters;
            filterResponse = loadedFilters.private(payload);
        }
        if (!filterResponse) {
            (0, metrics_1.incrementWebSocketRequestsTotal)(true, 'inbound-request');
            const reason = '[Websocket Flow][Blocked Request] Does not match any accept rule';
            logContext.error = 'Blocked by filter rules';
            logger_1.log.warn(logContext, reason);
            return emit({
                status: 401,
                body: {
                    message: 'blocked',
                    reason,
                    url: payload.url,
                },
            });
        }
        else {
            (0, metrics_1.incrementWebSocketRequestsTotal)(false, 'inbound-request');
            const preparedRequest = await (0, prepareRequest_1.prepareRequestFromFilterResult)(filterResponse, payload, logContext, options, connectionIdentifier, websocketConnectionHandler === null || websocketConnectionHandler === void 0 ? void 0 : websocketConnectionHandler.socketType);
            (0, metrics_1.incrementHttpRequestsTotal)(false, 'outbound-request');
            payload.streamingID
                ? await (0, requestsHelper_1.makePostStreamingRequest)(preparedRequest.req, emit, logContext)
                : await (0, requestsHelper_1.makeLegacyRequest)(preparedRequest.req, emit, logContext, options);
        }
    };
};
exports.forwardWebSocketRequest = forwardWebSocketRequest;
//# sourceMappingURL=forwardWebsocketRequest.js.map